# NATools - Package with parallel corpora tools
# Copyright (C) 2002-2004  Alberto Simes
#
# This package is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

package NAT::Lexicon;

use 5.006;
use strict;
use warnings;
require Exporter;

use NAT;
use MLDBM;
use Fcntl;
use Storable;
use Data::Dumper;

our @ISA = qw(Exporter);
our %EXPORT_TAGS = ( 'all' => [ qw() ] );
our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
our @EXPORT = qw();

our $VERSION = '0.02';

sub open {
  my $filename = shift;

  $filename = shift if $filename eq "NAT::Lexicon";

  return undef unless -f $filename;

  my $wlid = NAT::wlopen($filename);
  return undef if $wlid < 0;

  return bless +{ id => $wlid } #amen
}

sub close {
  my $self = shift;
  NAT::wlclose($self->{id});
}

sub word_from_id {
  my $self = shift;
  my $id = shift;
  return NAT::wlgetbyid($self->{id}, $id);
}

# NOTE: This creates the entry with 1 occurrence
sub add_word {
  my $self = shift;
  my $word = shift;
  return NAT::wladdword($self->{id}, lc($word));
}

sub enlarge {
  my ($self, $extrasize) = @_;
  return NAT::wlenlarge($self->{id}, $extrasize);
}

sub id_from_word {
  my $self = shift;
  my $word = shift;
  return NAT::wlgetbyword($self->{id}, lc($word));
}

sub sentence_to_ids {
  my $self = shift;
  my $sentence = shift;
  my @words = split /\s+/, $sentence;
  @words = map { $self->id_from_word($_) } @words;
  return \@words;
}

sub save {
  my ($self, $filename) = @_;
  NAT::wlsave($self->{id}, $filename);
}

# We will need something more to handle correct cases
sub ids_to_sentence {
  my $self = shift;
  return join(" ",map { $self->word_from_id($_) } @_);
}

sub size {
  my $self = shift;
  my $size = NAT::wlgetsize($self->{id});
  return $size?$size:undef;
}

sub occurrences {
  my $self = shift;
  my $size = NAT::wloccs($self->{id});
  return $size?$size:undef;
}

sub set_id_count {
  my ($self, $id, $count) = @_;
  return NAT::wlsetcountbyid($self->{id}, $id, $count);
}

sub id_count {
  my $self = shift;
  my $id = shift;
  return NAT::wlcountbyid($self->{id}, $id);
}

1;
__END__

=head1 NAME

NAT::Lexicon - Perl extension to encapsulate NATools Lexicon files

=head1 SYNOPSIS

  use NAT::Lexicon;

  $lex = NAT::Lexicon::open("file.lex");

  $word = $lex->word_from_id(2);

  $id = $lex->id_from_word("cavalo");

  @ids = $lex->sentence_to_ids("era uma vez um gato maltez");

  $sentence = $lex->ids_to_sentence(10,2,3,2,5,4,3,2,5);

  $lex->size;

  $lex->id_count(2);

  $lex->close;

=head1 DESCRIPTION

This module encapsulates the NATools Lexicon files, making them
accessible using Perl. The implementation is based on OO
philosophy. First, you must open a lexicon file using:

 $lex = NAT::Lexicon::open("lexicon.file.lex");

When you have all done, do not forget to close it. This makes some
memory frees, and is welcome for the process of opening new lexicon
files.

 $lex->close;

Lexicon files map words to identifiers and vice-versa. Its usage is
simple: use

  $lex->id_from_word($word)

to get an id for a word. Use

  $lex->word_from_id($id)

to get back the word from the id. If you need to make big quantities
of conversions to construct or parse a sentence use C<ids_to_sentence>
or C<sentence_to_ids> respectively.

=head1 SEE ALSO

See perl(1) and NATools documentation.

=head1 AUTHOR

Alberto Manuel Brandao Simoes, E<lt>albie@alfarrabio.di.uminho.ptE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright 2002-2004 by NATURA Project
http://natura.di.uminho.pt

This library is free software; you can redistribute it and/or modify
it under the GNU General Public License 2, which you should find on
parent directory. Distribution of this module should be done including
all NATools package, with respective copyright notice.

=cut
